/*
 * image class definition
 */
#ifndef F3DPIMAGE_H
#define F3DPIMAGE_H
#include <iostream>

//stdlib.h variable must be re-set to O_BINARY in MS Win Enviroment
//Include to each file useing stdin and/or stdout
#ifdef _MSC_VER
#include <io.h>
#include <fcntl.h>
#define MSC_SET_BINARY                                                  \
        int result = _setmode( _fileno( stdout ), _O_BINARY );          \
        if( result == -1 ) {                                            \
                cerr<<progname<<": cannot set stdout mode to binary"<<endl;\
                return 1;                                               \
        }                                                               \
        result = _setmode( _fileno( stdin ), _O_BINARY );               \
        if( result == -1 ) {                                            \
                cerr<<progname<<": cannot set stdin mode to binary"<<endl;\
                return 1;                                               \
        }
#else
#define MSC_SET_BINARY
#endif


/*
 * define the color triple
 * Components: red - x, green - y, blue - z
 */
class myColor {
public:
    /// x component
    float x;
    /// y component
    float y;
    /// z component
    float z;

    /// default constructor, not initialized
    myColor(void) {}

    /// construct the vector from 3 values 
    explicit myColor(float X, float Y, float Z): x(X), y(Y), z(Z) {}

    /// scalar multiplication
    myColor operator*= (double A)
    {
        x *= A;  y *= A;  z *= A;
        return *this;
    }

};

class myImage
{
private:
	myColor *img;	// the image data
	int nx, ny;	// dimensions

public:
	/** Constructor: create the image
	 * @param mono if true, graylevel image is created
	 */
	myImage(int _nx, int _ny, bool _mono=false): nx(_nx), ny(_ny) 
	{ create(); }

	/// Destructor
	~myImage(void) {destroy();}

	/// copy constructor
	myImage(myImage const &other) { copy(other); }

	/// overloaded assignment
	myImage const& operator=(myImage const &other)
	{
		if(this != &other){ 
			delete img; img = 0;
			copy(other); 
		}
		return *this;
	}

	// set color
	void set(int x, int y, myColor &c) { *(img+y*nx+x) = c;}

	// set color by a scalat value
	void set(int x, int y, float v) { *(img+y*nx+x) = myColor(v, v, v);}

	// get color
	myColor getColor(int x, int y) { return *(img+y*nx+x);}

	// get color
	float getScalar(int x, int y) { return (img+y*nx+x)->x;}
	
	/** normalize the image
	 * @param val value to multiply all pixels with
	 */
	void norm(float val){
		for (int i=0; i<nx*ny; i++) *(img+i) *= val;
	}

	/** save the image as a ppm file
	 * Overloaded << operator for C++-style output
	 */
	friend std::ostream& operator<< (std::ostream& s, const myImage& im)
	{
		s << "P6" << std::endl <<  im.nx << " " << im.ny << std::endl << 255 <<std::endl;
		unsigned char c;
		int i;
		for (i=0; i<im.nx*im.ny; i++) {
			c =  (unsigned char)(im.img+i)->x; s.write((char*)&c, 1);
			c =  (unsigned char)(im.img+i)->x; s.write((char*)&c, 1);
			c =  (unsigned char)(im.img+i)->x; s.write((char*)&c, 1);
		}
		return s;
	}


private:
	///  copy the image
	void copy(myImage const &other)
	{
		nx = other.nx; ny = other.ny;
		create();
		memcpy(img, other.img, sizeof(myColor) * nx * ny);
	}

	/// create the band
	void create(void) { 
		img = new myColor [nx*ny]; 
		if(!img){
			std::cerr<<"myImage:: image allocation error"<<std::endl;
			exit(1);
		}
	}

	/// destroy the image
	void destroy(void)
	{
		if(img) { delete [] img; img=0; }
	}
};
#endif //F3DPIMAGE_H
